/*
 *   This file is part of Dianara
 *   Copyright 2012  JanKusanagi <janjabber@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .
 */

#include "timeline.h"

TimeLine::TimeLine(DiasporaController *diasporaController, QWidget *parent) :  QWidget(parent)
{
    this->dController = diasporaController;

    mainLayout = new QVBoxLayout();
    mainLayout->setAlignment(Qt::AlignTop);


    QMap<QString,QString> demoPostData;
    demoPostData.insert("author", "DemoAuthor");
    demoPostData.insert("text",   "## ***Demo Post***\n"
                                  "Press **F5** or *Update timelines* in the menu.\n\n"
                                  "Upon startup, timeline should update automatically.");
    demoPostData.insert("createdAt", "1970-01-01T01:02:03Z");
    demoPostData.insert("likesCount", "7");
    demoPostData.insert("commentsCount", "256");
    demoPostData.insert("resharesCount", "1024");

    mainLayout->addWidget(new Post(dController, demoPostData, this));

    this->setLayout(mainLayout);

    qDebug() << "TimeLine created";
}


TimeLine::~TimeLine()
{
    qDebug() << "TimeLine destroyed";
}



/*
 * Remove all widgets (Post *) from the timeline
 *
 */
void TimeLine::clearTimeLineContents()
{
    QList<QWidget*> previousPostList;

    // Build list of widgets contained in the layout
    for (int counter = 0; counter < mainLayout->count(); ++counter)
    {
        qDebug() << "Adding for deletion:" << mainLayout->itemAt(counter)->widget();

        previousPostList.append(mainLayout->itemAt(counter)->widget());
    }

    qDebug() << "List to delete:" << previousPostList;


    foreach (QWidget *oldPost, previousPostList)
    {
        //delete mainLayout->itemAt(counter)->widget();
        //mainLayout->itemAt(counter)->widget()->deleteLater();

        qDebug() << "will deleteLater:" << oldPost;
        oldPost->deleteLater(); // delete should be safe too, but stil...
    }
}



/************************ SLOTS **************************/


void TimeLine::setTimeLineContents(QVariantList postList)
{
    qDebug() << "TimeLine::setTimeLineContents()";
    //qDebug() << postList;


    // Remove all previous posts in timeline
    qDebug() << "Removing previous posts from timeline";
    this->clearTimeLineContents();


    // Fill timeline with new contents
    foreach (QVariant singlePost, postList)
    {
        if (singlePost.type() == QVariant::Map)
        {
            qDebug() << "Parsed a Map:";
            QVariantMap singlePostMap = singlePost.toMap();
            //qDebug() << "Keys:" << singlePostMap.keys();

            QString postAuthor;
            QString postAuthorAvatar;
            QString postText;
            QString postType;
            QString postCreatedAt;
            QString postRootAuthor;
            QString postLikesCount;
            QString postCommentsCount;
            QString postResharesCount;

            postAuthor = singlePostMap.value("author").toMap().value("name").toString();
            postAuthorAvatar = singlePostMap.value("author").toMap().value("avatar").toMap().value("medium").toString();

            postCreatedAt = singlePostMap.value("created_at").toString();

            postType = singlePostMap.value("post_type").toString();
            qDebug() << "post type:" << postType;
            if (postType == "StatusMessage")
            {
                postText = singlePostMap.value("text").toString();
            }
            if (postType == "Reshare")
            {
                postRootAuthor = singlePostMap.value("root").toMap().value("author").toMap().value("name").toString();
                postText.append(singlePostMap.value("text").toString());
            }

            postLikesCount = singlePostMap.value("likes_count").toString();
            postCommentsCount = singlePostMap.value("comments_count").toString();
            postResharesCount = singlePostMap.value("reshares_count").toString();


            QMap<QString,QString> postData;
            postData.insert("author",        postAuthor);
            postData.insert("authorAvatar",  postAuthorAvatar);
            postData.insert("text",          postText);
            postData.insert("createdAt",     postCreatedAt);
            postData.insert("postType",      postType);
            postData.insert("rootAuthor",    postRootAuthor);
            postData.insert("likesCount",    postLikesCount);
            postData.insert("commentsCount", postCommentsCount);
            postData.insert("resharesCount", postResharesCount);


            this->mainLayout->addWidget(new Post(dController, postData, this));
        }
        else
        {
            qDebug() << "Expected a Map, got something else";
            qDebug() << postList;
        }

    }
    qDebug() << "setTimeLineContents() /END";
}




